'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TraceKit = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = initErrorTransformer;

var _tracekit = require('tracekit');

var _tracekit2 = _interopRequireDefault(_tracekit);

require('whatwg-fetch');

var _transport = require('./transport');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; } /**
                                                                                                                                                                                                                              * The purpose of this file is to provide redux-logger middleware for error logging
                                                                                                                                                                                                                              * To be applied as errorTransformer, or even actionTransformer
                                                                                                                                                                                                                              * @exports redux-logger compatible error logging middleware
                                                                                                                                                                                                                              * @export TraceKit, to avoid adding dependency to customer's code
                                                                                                                                                                                                                              **/

var defaultOptions = {
  stackTraceLimit: Error.stackTraceLimit,
  linesOfContext: 15
};

/**
 * @param rethrowErrorHandler - handler for rethrown errors
 * @returns Report, wrapped TraceKit.report method which no longer rethrows an error
**/
function safeWrapReport(rethrowErrorHandler) {
  function report(ex) {
    try {
      _tracekit2.default.fallback(ex);
    } catch (e) {
      rethrowErrorHandler(ex);
    }
  }
  report.subscribe = _tracekit2.default.report.subscribe;
  report.unsubscribe = _tracekit2.default.report.unsubscribe;
  _tracekit2.default.fallback = _tracekit2.default.report;
  return report;
}

/**
 * @param logServerUrl - string, fetch-compatible logger instance URL
 * @param transport - object, {
 *   send: function that accepts (payload, fetchOptions) and returns Promise, {
 *     payload: string or json object, report
 *     fetchOptions: object, options that transport.send accepts { headers, method, etc }
 *   }
 *   successHandler: function to be called on successful report,
 *                   takes response text, return is ignored,
 *   failedTryHandler: function to be called on transport.send error,
 *                     takes {
 *                       error: Error object thrown inside of transfer.send,
 *                       payload: string or json object, report before middleware was applied,
 *                       transportOpts: reference to the transport object,
 *                       attempt: # of try to send report,
 *                     }, return is ignored, by default tries to repeat request,
 *   failedReportHandler: function to be called on report general failure,
 *                        takes Error object, return is ignored,
 *   payloadMiddleware: function to be applied on payload, to attach store state, time, etc,
 *   retryCount: number # of max retries, by-default 2, set to 0 if you don't want retries
 *   retryTimeout: number in ms, by-default 60 seconds,
 * }
 * @param options - object {
 *   stackTraceLimit: number, max call stack size, applied globally on Error object,
 *   linesOfContext: number, should be odd and >=3, by-default 15,
 *   rethrowError: boolean, tells to either monkey-patch TraceKit.report method or no, false by-def,
 *   otherTraceKitOptions can be any options that TraceKit understands
 * }
 * @returns function, redux-logger compatible middleware for messages
 **/
function initErrorTransformer(logServerUrl) {
  var transport = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  var _Object$assign = Object.assign(defaultOptions, options),
      stackTraceLimit = _Object$assign.stackTraceLimit,
      rethrowErrorHandler = _Object$assign.rethrowErrorHandler,
      linesOfContext = _Object$assign.linesOfContext,
      otherTraceKitOptions = _objectWithoutProperties(_Object$assign, ['stackTraceLimit', 'rethrowErrorHandler', 'linesOfContext']);

  var mergedTransport = Object.assign((0, _transport.getDefaultTransport)(logServerUrl), transport);

  Error.stackTraceLimit = stackTraceLimit;
  Object.assign(_tracekit2.default, _extends({
    fetchContext: !!linesOfContext,
    linesOfContext: linesOfContext
  }, otherTraceKitOptions));

  if (rethrowErrorHandler) {
    _tracekit2.default.report = safeWrapReport(rethrowErrorHandler);
  }

  function listener(payload) {
    return (0, _transport.sendReport)(payload, mergedTransport);
  }

  _tracekit2.default.report.subscribe(listener);

  return listener;
}

var TraceKit = exports.TraceKit = _tracekit2.default;